-----------------------------------------------------------------------------------------------
-- Client Lua Script
-- Copyright (c) Frosthaven. All rights reserved
-----------------------------------------------------------------------------------------------

require "Apollo"
require "Window"
require "Spell"
require "CombatFloater"
require "GameLib"
require "Unit"

-----------------------------------------------------------------------------------------------
-- Configuration
-----------------------------------------------------------------------------------------------
local FMCT = {
	meta   = {
		addonName    = 'FrostMod CombatText',
		addonAuthor  = 'Frosthaven',
		addonVersion = '0.3 Dev-Release',
		addonSlash   = 'fmct',
		addonBadge   = 'FrostModSprites:uiBadge',
	},
	dev = {
		debug        = true,
		spamUnitType = false,
	},
	--config and defaults are loaded by code
	config   = {},
	defaults = {},
}

-----------------------------------------------------------------------------------------------
-- Loaders & Savers
-----------------------------------------------------------------------------------------------
function FMCT:GetDefaults()
return {
		allowOverNameplates    = true,
		allowCustomHarvestText = true,
		autoTargetOnOutgoing   = true,
		preventOutClipping     = true,
		customHarvestText      = "Harvesting",
		style = {
			-- outgoing combat text --------------------------------------------
			--------------------------------------------------------------------
			outgoing = {
				constraints = {
					hit       = { font = "CRB_FloaterHuge", size = "0.6", duration = "0.2", },
					crit      = { font = "CRB_FloaterHuge", size = "0.7", duration = "0.9", },
					important = { font = "CRB_FloaterHuge", size = "0.6", duration = "1.5", },
				},
				harvest    = { hit = "ff0071", crit = "ff5da5", },
				damage     = { hit = "e5feff", crit = "fffb93", },
				pet        = { hit = "ffcdb3", crit = "ffcdb3", },
				absorb     = { hit = "ff600a", crit = "ff600a", },
				vulnerable = { hit = "f5a2ff", crit = "fffb93", },
				heal       = { hit = "b0ff6a", crit = "cdffa0", },
				shield     = { hit = "6afff3", crit = "c9fffb", },
				miss       = { hit = "ff600a", crit = "ff600a", },
				ccSuccess  = { hit = "ff600a", crit = "ff600a", },
				ccFail     = { hit = "ff600a", crit = "ff600a", },
				ccList = {
					[Unit.CodeEnumCCState.Stun]      = true, [Unit.CodeEnumCCState.Sleep]   = true, [Unit.CodeEnumCCState.Root]           = true,
					[Unit.CodeEnumCCState.Disarm]    = true, [Unit.CodeEnumCCState.Silence] = true, [Unit.CodeEnumCCState.Polymorph]      = true,
					[Unit.CodeEnumCCState.Fear]      = true, [Unit.CodeEnumCCState.Hold]    = true, [Unit.CodeEnumCCState.Knockdown]      = true,
					[Unit.CodeEnumCCState.Disorient] = true, [Unit.CodeEnumCCState.Disable] = true, [Unit.CodeEnumCCState.Taunt]          = true,
					[Unit.CodeEnumCCState.DeTaunt]   = true, [Unit.CodeEnumCCState.Blind]   = true, [Unit.CodeEnumCCState.Knockback]      = true,
					[Unit.CodeEnumCCState.Pushback]  = true, [Unit.CodeEnumCCState.Pull]    = true, [Unit.CodeEnumCCState.PositionSwitch] = true,
					[Unit.CodeEnumCCState.Tether]    = true, [Unit.CodeEnumCCState.Snare]   = true, [Unit.CodeEnumCCState.Interrupt]      = true,
					[Unit.CodeEnumCCState.Daze]      = true, [Unit.CodeEnumCCState.Subdue]  = true,
				},
			},
			-- incoming combat text --------------------------------------------
			--------------------------------------------------------------------
			incoming = {
				constraints = {
					hit       = { font = "CRB_FloaterHuge", size = "0.8", duration = "1.0", },
					crit      = { font = "CRB_FloaterHuge", size = "0.9", duration = "1.0", },
					important = { font = "CRB_FloaterHuge", size = "0.8", duration = "1.0", },
				},
				damage     = { hit  = "ff3d3d", crit = "ff0000", },
				pet        = { hit  = "ff3d3d", crit = "ff0000", },
				absorb     = { hit  = "ff600a", crit = "ff600a", },
				vulnerable = { hit  = "ff7878", crit = "ff0000", },
				heal       = { hit  = "b0ff6a", crit = "00ff00", },
				shield     = { hit  = "38acff", crit = "85ccff", },
				miss       = { hit  = "b0b0b0", crit = "b0b0b0", },
				ccSuccess  = { hit  = "ff3d3d", crit = "ff0000", },
				ccFail     = { hit  = "b0b0b0", crit = "b0b0b0", },
				ccList = {
					[Unit.CodeEnumCCState.Stun]      = true, [Unit.CodeEnumCCState.Sleep]   = true, [Unit.CodeEnumCCState.Root]           = true,
					[Unit.CodeEnumCCState.Disarm]    = true, [Unit.CodeEnumCCState.Silence] = true, [Unit.CodeEnumCCState.Polymorph]      = true,
					[Unit.CodeEnumCCState.Fear]      = true, [Unit.CodeEnumCCState.Hold]    = true, [Unit.CodeEnumCCState.Knockdown]      = true,
					[Unit.CodeEnumCCState.Disorient] = true, [Unit.CodeEnumCCState.Disable] = true, [Unit.CodeEnumCCState.Taunt]          = true,
					[Unit.CodeEnumCCState.DeTaunt]   = true, [Unit.CodeEnumCCState.Blind]   = true, [Unit.CodeEnumCCState.Knockback]      = true,
					[Unit.CodeEnumCCState.Pushback]  = true, [Unit.CodeEnumCCState.Pull]    = true, [Unit.CodeEnumCCState.PositionSwitch] = true,
					[Unit.CodeEnumCCState.Tether]    = true, [Unit.CodeEnumCCState.Snare]   = true, [Unit.CodeEnumCCState.Interrupt]      = true,
					[Unit.CodeEnumCCState.Daze]      = true, [Unit.CodeEnumCCState.Subdue]  = true,
				},
			},
		}
	}
end

function FMCT:OnSave(eType)
	-- eType is one of:
	-- GameLib.CodeEnumAddonSaveLevel.General
	-- GameLib.CodeEnumAddonSaveLevel.Account
	-- GameLib.CodeEnumAddonSaveLevel.Realm
	-- GameLib.CodeEnumAddonSaveLevel.Character
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		return g_FMCT.config
	end
end

function FMCT:OnRestore(eType, tSavedData)
	-- this is only called when there is data to restore
	if eType == GameLib.CodeEnumAddonSaveLevel.Character then
		--[[
			using global names for our config and saved
			data allows us to run deep comparisons via _G

			each setting, nested or not, will be restored
			only if the following check out:

			- the saved setting exists in the main config
			- the saved setting is not nil
			- the saved setting is of the same type()
			- the saved setting is different

			personal note: non-nested configs are easier...
		--]]
		g_FMCT.saved = tSavedData

		local currentpath = "g_FMCT.config"
		local savedpath   = "g_FMCT.saved"
		_deepRestore(currentpath, savedpath)
	end
end

-----------------------------------------------------------------------------------------------
-- Cache
-----------------------------------------------------------------------------------------------
local cache = {
	lastOffset1    = 0,
	lastOffset2    = 0,
	lastDamageTime = 0,
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function FMCT:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

function FMCT:Init()
	g_FMCT = self
	g_saved = {}
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"uiMapper:0.7",
		"FloatText",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- Save & Restore
-----------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------
-- Load Functions
-----------------------------------------------------------------------------------------------
function FMCT:OnLoad()
	-- load default settings -------------------------------------------
	--------------------------------------------------------------------
	self.config   = self:GetDefaults()
	self.defaults = self:GetDefaults()

	-- load sprite sheet -----------------------------------------------
	--------------------------------------------------------------------
	Apollo.LoadSprites("textures/sprites.xml", "FrostModSprites")

	-- register detours ------------------------------------------------
	--------------------------------------------------------------------
	saved = {}
	CRB_FloatText = Apollo.GetAddon("FloatText")

	-- detours: FloatText ----------------------------------------------
	--------------------------------------------------------------------
	saved.CRB_FloatText = {}
	--saved.CRB_FloatText.OnDamageOrHealing       = CRB_FloatText.OnDamageOrHealing
	CRB_FloatText.OnDamageOrHealing               = detour_OnDamageOrHealing
	--saved.CRB_FloatText.OnPlayerDamageOrHealing = CRB_FloatText.OnPlayerDamageOrHealing
	CRB_FloatText.OnPlayerDamageOrHealing         = detour_OnPlayerDamageOrHealing
	--saved.CRB_FloatText.OnCombatLogCCState 	  = CRB_FloatText.OnCombatLogCCState
	CRB_FloatText.OnCombatLogCCState       		  = detour_OnCombatLogCCState
	--saved.CRB_FloatText.OnMiss                  = CRB_FloatText.OnMiss
	CRB_FloatText.OnMiss                          = detour_OnMiss

	-- initialize the ui -----------------------------------------------
	--------------------------------------------------------------------
	local uiMapper = Apollo.GetPackage("uiMapper:0.7").tPackage
	self.ui  = uiMapper:new({
		container = g_FMCT.config,
		defaults  = g_FMCT.defaults,
		name      = "FrostMod CombatText",
		author    = "Frosthaven",
		version   = "0.5 dev-release",
		slash     = "fmct"
	}):build(function(ui)
		_BuildUI(ui)
	end)
end

function FMCT:ValueChanged(from, to, wndHandle)

end

-----------------------------------------------------------------------------------------------
-- Detour Functions
-----------------------------------------------------------------------------------------------
function detour_OnCombatLogCCState(self, tEventArgs)
	--todo
	-- dont process if invalid/disabled --------------------------------
	--------------------------------------------------------------------
	if tEventArgs.eResult == nil or not Apollo.GetConsoleVariable("ui.showCombatFloater") then
		return
	end

	-- reroute stray player numbers to player floater ------------------
	--------------------------------------------------------------------
	if GameLib.IsControlledUnit(tEventArgs.unitTarget) then
		detour_OnCombatLogCCStatePlayer(self, tEventArgs)
		return
	end

	-- prepare cc listings ---------------------------------------------
	--------------------------------------------------------------------
	local arCCWhitelist = g_FMCT.config.style.outgoing.ccList
	local nOffsetState  = tEventArgs.eState

	-- handle the event ------------------------------------------------
	--------------------------------------------------------------------
	if tEventArgs.eResult == CombatFloater.CodeEnumCCStateApplyRulesResult.Ok then -- CC applied
		strMessage = tEventArgs.strState
		if arCCWhitelist[nOffsetState] == 1 then -- make sure it's one we want to show
			strLabel   = "ccSuccess"
		else
			return
		end
	elseif tEventArgs.eResult == CombatFloater.CodeEnumCCStateApplyRulesResult.Target_Immune then
		strMessage = Apollo.GetString("FloatText_Immune")
		strLabel   = "ccFail"
	elseif tEventArgs.eResult == CombatFloater.CodeEnumCCStateApplyRulesResult.Target_InfiniteInterruptArmor then
		strMessage = Apollo.GetString("FloatText_InfInterruptArmor")
		strLabel   = "ccFail"
	elseif tEventArgs.eResult == CombatFloater.CodeEnumCCStateApplyRulesResult.Target_InterruptArmorReduced then -- use with interruptArmorHit
		strMessage = String_GetWeaselString(Apollo.GetString("FloatText_InterruptArmor"), tEventArgs.nInterruptArmorHit)
		strLabel   = "ccSuccess"
	elseif tEventArgs.eResult == CombatFloater.CodeEnumCCStateApplyRulesResult.DiminishingReturns_TriggerCap and tEventArgs.strTriggerCapCategory ~= nil then
		strMessage = Apollo.GetString("FloatText_CC_DiminishingReturns_TriggerCap").." "..tEventArgs.strTriggerCapCategory
		strLabel   = "ccFail"
	else
		-- all invalid messages
		return
	end

	detour_OnDamageOrHealing(self, tEventArgs.unitCaster, tEventArgs.unitTarget, false, false, false, false, false, false, strLabel, strMessage)

end

function detour_OnCombatLogCCStatePlayer(self, tEventArgs)
	--todo
end

function detour_OnMiss(self, unitCaster, unitTarget, eMissType)
	--route misses through combat damage
	detour_OnDamageOrHealing(self, unitCaster, unitTarget, false, false, false, false, false, false, "miss", eMissType)
end

function detour_OnDamageOrHealing(self, unitCaster, unitTarget, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical, nSpellName, xLabel, xData)
	-- dont process if invalid/disabled --------------------------------
	--------------------------------------------------------------------
	if unitTarget == nil or not Apollo.GetConsoleVariable("ui.showCombatFloater") or nDamage == nil then
		return
	end

	-- reroute stray player numbers to incoming player -----------------
	--------------------------------------------------------------------
	if GameLib.IsControlledUnit(unitTarget)
		or unitTarget == GameLib.GetPlayerMountUnit()
		or unitTarget == GameLib.GetPlayerUnit() then
		detour_OnPlayerDamageOrHealing(self, unitTarget, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical, xLabel, xData)
		return
	end

	-- setup anchor positioning for important notices ------------------
	--------------------------------------------------------------------
	if  unitTarget ~= nil and unitTarget:GetOverheadAnchor() ~= nil and unitTarget:GetOverheadAnchor().y ~= nil and GameLib.GetPlayerUnit() then 
		nTargetOverheadAnchor = unitTarget:GetOverheadAnchor().y
		nPlayerOverheadAnchor = GameLib.GetPlayerUnit():GetOverheadAnchor().y
	else
		nTargetOverheadAnchor = Apollo.GetDisplaySize().nHeight/3
		nPlayerOverheadAnchor = GameLib.GetPlayerUnit():GetOverheadAnchor().y
	end
	local importantAnchor =  g_FMCT:FloaterOffsetAdjustment(nPlayerOverheadAnchor,nTargetOverheadAnchor, true)
	local anchor          =  g_FMCT:FloaterOffsetAdjustment(nPlayerOverheadAnchor,nTargetOverheadAnchor, false)

	-- handle style flags ----------------------------------------------
	--------------------------------------------------------------------
	local flags       = g_FMCT:GetDamageFlags(unitTarget, unitCaster, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical, xLabel, xData)
	local style       = g_FMCT:GetStyleByFlags(flags,"outgoing")
	local tTextOption = g_FMCT:GetDefaultTextSettings()

	-- font constraints ------------------------------------------------
	--------------------------------------------------------------------
	tTextOption.strFontFace = style.constraints.font
	local size              = tonumber(style.constraints.size)
	local initialSize       = tonumber(style.constraints.size) * 1.2
	local duration          = tonumber(style.constraints.duration)
	local baseColor, flashColor, opacity
	if type(style.color) == 'string' then
		baseColor         = "0x" .. style.color
		flashColor        = "0x" .. style.color
		opacity           = 1
	else
		baseColor         = "0x" .. g_FMCT.ui:RGBtoHEX(style.color)
		flashColor        = "0x" .. g_FMCT.ui:RGBtoHEX(style.color)
		opacity           = style.color.a
	end

	if flags.crit or flags.important then
		initialSize = tonumber(style.constraints.size) * 1.5
		flashColor = "0xffffff"
	end

	-- offset and direction --------------------------------------------
	--------------------------------------------------------------------
	--we *try* to randomize the text if its too close to the previous two entries
	local boundary = 50
	local range    = 360
	local offset   = math.random(0, range)  -- disabled randomization of offset by setting min/max to 0,0
	if offset <= (cache.lastOffset1 + boundary) and offset >= (cache.lastOffset1 - boundary)
	or offset <= (cache.lastOffset2 + boundary) and offset >= (cache.lastOffset2 - boundary) then
		offset = math.random(0, range)  -- disabled randomization of offset by setting min/max to 0,0
	end
	cache.lastOffset1            = cache.lastOffset2
	cache.lastOffset2            = offset
	tTextOption.fOffsetDirection = offset
	tTextOption.fOffset          = math.random(10, 80)/100
	tTextOption.bShowOnTop       = false

	-- location and collision ------------------------------------------
	--------------------------------------------------------------------
	tTextOption.eCollisionMode = CombatFloater.CodeEnumFloaterCollisionMode.IgnoreCollision  --Horizontal--Vertical--IgnoreCollision
	tTextOption.eLocation      = anchor

	-- option overrides ------------------------------------------------
	--------------------------------------------------------------------
	tTextOption.bShowOnTop = g_FMCT.config.allowOverNameplates

	-- animation -------------------------------------------------------
	--------------------------------------------------------------------
	tTextOption.fVelocityMagnitude = 1

	tTextOption.arFrames =
	{
		--pop in small
		[1] = { fScale = initialSize * .5, fTime = 0,   fAlpha = opacity, nColor = flashColor, fVelocityDirection = 0, fVelocityMagnitude = 0, },
		--quick growth
		[2] = { fScale = initialSize,      fTime = 0.1,	fAlpha = opacity, nColor = flashColor, fVelocityDirection = 0, fVelocityMagnitude = 0, },
	}

	if flags.crit or flags.important then
		-- normal size
		tTextOption.arFrames[3] = { fScale = size, fTime = 0.3,            fAlpha = opacity,  nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 0, }
		-- flying up
		tTextOption.arFrames[4] = {	fScale = size, fTime = 0.3 + duration, fAlpha = opacity,  nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 0, }
		-- fading out
		tTextOption.arFrames[5] = {	fScale = size, fTime = 0.4 + duration, fAlpha = opacity*0.75, nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 5, }
		-- gone
		tTextOption.arFrames[6] = {	fScale = size, fTime = 0.5 + duration, fAlpha = 0,    nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 7, }
	else
		-- normal size
		tTextOption.arFrames[3] = { fScale = size, fTime = 0.3,            fAlpha = opacity,  nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 0, }
		--flying up
		tTextOption.arFrames[4] = { fScale = size, fTime = 0.3 + duration, fAlpha = opacity*0.5,  nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 0, }
		--fading out
		tTextOption.arFrames[5] = { fScale = size, fTime = 0.4 + duration, fAlpha = opacity*0.5,  nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 5, }
		--gone
		tTextOption.arFrames[6] = { fScale = size, fTime = 0.5 + duration, fAlpha = 0,    nColor = baseColor, fVelocityDirection = 0, fVelocityMagnitude = 7, }
	end
	if not flags.heal then
		cache.lastDamageTime = GameLib.GetGameTime()
	end

	-- override layout for important data ------------------------------
	--------------------------------------------------------------------
	if (flags.important) then
		tTextOption.eLocation      = importantAnchor
		tTextOption.eCollisionMode = CombatFloater.CodeEnumFloaterCollisionMode.horizontal
		tTextOption.fOffset          = 0
		tTextOption.fOffsetDirection = 0
	end

	-- show the text ---------------------------------------------------
	--------------------------------------------------------------------
	if (xLabel == "miss") then
		local strText = (xData == GameLib.CodeEnumMissType.Dodge) and Apollo.GetString("CRB_Dodged") or Apollo.GetString("CRB_Blocked")
		CombatFloater.ShowTextFloater( unitTarget, strText, tTextOption )
	elseif (xLabel == "ccSuccess" or xLabel == "ccFail") then
		CombatFloater.ShowTextFloater( unitTarget, xData, tTextOption )
	elseif flags.petIn and not flags.heal and not flags.shield then
		local petDamage

		if nShieldDamaged > 0 then
			if nDamage > 0 then
				--shield damage and regular damage, so combine the numbers
				petDamage = nDamage + nShieldDamaged
			else
				--only shield is damaged
				petDamage = nShieldDamaged
			end
		else
			--only regular damage
			petDamage = nDamage
		end

		CombatFloater.ShowTextFloater( unitTarget, String_GetWeaselString(petDamage), tTextOption )
	elseif flags.absorb then -- secondary "if" so we don't see absorption and "0"
		-- CombatFloater.ShowTextFloater( unitTarget, String_GetWeaselString(Apollo.GetString("FloatText_Absorbed"), nAbsorptionAmount), tTextOption )
		CombatFloater.ShowTextFloater( unitTarget, String_GetWeaselString(nAbsorptionAmount), tTextOption )

		if nDamage > 0 then
			if flags.heal then
				CombatFloater.ShowTextFloater( unitTarget, String_GetWeaselString(Apollo.GetString("FloatText_PlusValue"), nDamage), tTextOption )
			else
				CombatFloater.ShowTextFloater( unitTarget, nDamage, 0, tTextOption )
			end
		end
	elseif flags.heal or flags.shield then
		CombatFloater.ShowTextFloater( unitTarget, String_GetWeaselString(Apollo.GetString("FloatText_PlusValue"), nDamage), tTextOption )
	else
		--show harvesting text instead of damage
		if flags.harvest and g_FMCT.config.allowCustomHarvestText then
			nDamage = g_FMCT.config.customHarvestText
		end

		if nShieldDamaged and nShieldDamaged > 0 then
			CombatFloater.ShowTextFloater(unitTarget, nDamage, 0, tTextOption)
		else
			CombatFloater.ShowTextFloater(unitTarget, nDamage, 0, tTextOption)
		end
	end

	if not GameLib.GetPlayerUnit():GetTarget()
		and not flags.heal
		and not flags.shield
		and not flags.harvest
		and g_FMCT.config.autoTargetOnOutgoing then
		GameLib.SetTargetUnit(unitTarget)
	end
end

function detour_OnPlayerDamageOrHealing(self, unitPlayer, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical, xLabel, xData)
	-- dont process if invalid/disabled --------------------------------
	--------------------------------------------------------------------
	if unitPlayer == nil or not Apollo.GetConsoleVariable("ui.showCombatFloater") or nDamage == nil then
		return
	end
	-- setup -----------------------------------------------------------
	--------------------------------------------------------------------
	local bShowFloater = true
	local tTextOption = self:GetDefaultTextOption()
	local tTextOptionAbsorb = self:GetDefaultTextOption()
	tTextOption.arFrames = {}
	tTextOptionAbsorb.arFrames = {}
	local nStallTime = .4
	
	-- handle style flags ----------------------------------------------
	--------------------------------------------------------------------
	local flags       = g_FMCT:GetDamageFlags(unitPlayer, unitCaster, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical, xLabel, xData)
	local style       = g_FMCT:GetStyleByFlags(flags,"incoming")
	local tTextOption = g_FMCT:GetDefaultTextSettings()
	
	-- font constraints ------------------------------------------------
	--------------------------------------------------------------------
	tTextOption.strFontFace        = style.constraints.font
	local fMaxSize                 = tonumber(style.constraints.size)
	local fMaxDuration             = tonumber(style.constraints.duration)
	local flashSizeMultiplier      = 1.5
	tTextOption.fVelocityMagnitude = 1

	local baseColor, opacity
	if type(style.color) == 'string' then
		baseColor         = "0x" .. style.color
		opacity           = 1
	else
		baseColor         = "0x" .. g_FMCT.ui:RGBtoHEX(style.color)
		opacity           = style.color.a
	end

	nBaseColor          = "0x" .. style.color
	flashColor          = "0xffffff"

	-- offset and direction --------------------------------------------
	--------------------------------------------------------------------
	tTextOption.fOffsetDirection   = 180
	tTextOption.fOffset            = 0.8

	-- location and collision ------------------------------------------
	--------------------------------------------------------------------
	tTextOption.eCollisionMode = CombatFloater.CodeEnumFloaterCollisionMode.Horizontal  --Horizontal--Vertical--IgnoreCollision
	tTextOption.eLocation      = CombatFloater.CodeEnumFloaterLocation.Head

	-- animation -------------------------------------------------------
	--------------------------------------------------------------------
	nStallTime = .4
	tTextOption.arFrames =
	{
		[1] = {fScale = fMaxSize * .75,	fTime = 0,									nColor = flashColor,	fVelocityDirection = 0,		fVelocityMagnitude = 0,},
		[2] = {fScale = fMaxSize * 1.5,	fTime = 0.05,								nColor = flashColor,	fVelocityDirection = 0,		fVelocityMagnitude = 0,},
		[3] = {fScale = fMaxSize,		fTime = 0.1,				fAlpha = opacity,	nColor = nBaseColor,},
		[4] = {							fTime = 0.3 + nStallTime,	fAlpha = opacity,							fVelocityDirection = 180,	fVelocityMagnitude = 2.5,},
		[5] = {							fTime = 1.2 + nStallTime,	fAlpha = opacity*0.3,							fVelocityDirection = 180,},
	}

	-- show the text ---------------------------------------------------
	--------------------------------------------------------------------
	if (xLabel == "miss") then
		--dodge/block/deflect
		local strText = (eMissType == GameLib.CodeEnumMissType.Dodge) and Apollo.GetString("CRB_Dodged") or Apollo.GetString("CRB_Blocked")
		CombatFloater.ShowTextFloater( unitPlayer, strText, tTextOption )
	elseif (xLabel == "ccSuccess" or xLabel == "ccFail") then
		--crowd control
		tTextOption.arFrames =
		{
			[1] = {fScale = 2.0,	fTime = 0,		fAlpha = opacity,	nColor = nFlashColor,},
			[2] = {fScale = 0.7,	fTime = 0.15,	fAlpha = opacity,},
			[3] = {					fTime = 0.5,					nColor = nBaseColor,},
			[4] = {fScale = 0.7,	fTime = 1.1,	fAlpha = opacity,										fVelocityDirection = 0,	fVelocityMagnitude = 5,},
			[5] = {					fTime = 1.3,	fAlpha = 0.0,										fVelocityDirection = 0,},
		}
		CombatFloater.ShowTextFloater( unitPlayer, xData, tTextOption )
	elseif flags.absorb then
		--absorb
		CombatFloater.ShowTextFloater( unitPlayer,  String_GetWeaselString(nAbsorptionAmount), tTextOption )

		if nDamage > 0 then
			if flags.heal then
				--heal + absorb
				CombatFloater.ShowTextFloater( unitPlayer, String_GetWeaselString(Apollo.GetString("FloatText_PlusValue"), nDamage), tTextOption )
			else
				--damage + absorb
				CombatFloater.ShowTextFloater( unitPlayer, nDamage, 0, tTextOption )
			end
		end
	elseif flags.heal or flags.shield then
		--heal or shield heal
		CombatFloater.ShowTextFloater( unitPlayer, String_GetWeaselString(Apollo.GetString("FloatText_PlusValue"), nDamage), tTextOption )
	else
		if nShieldDamaged and nShieldDamaged > 0 then
			CombatFloater.ShowTextFloater( unitPlayer, nDamage, 0, tTextOption )
		else
			CombatFloater.ShowTextFloater( unitPlayer, nDamage, 0, tTextOption )
		end
	end
end

-----------------------------------------------------------------------------------------------
-- Helper Functions
-----------------------------------------------------------------------------------------------
function FMCT:GetDefaultTextSettings()
	local tTextOption = {
		strFontFace 				= "CRB_FloaterLarge",
		fDuration 					= 2,
		fScale 						= 0.9,
		fExpand 					= 1,
		fVibrate 					= 0,
		fSpinAroundRadius 			= 0,
		fFadeInDuration 			= 0,
		fFadeOutDuration 			= 0,
		fVelocityDirection 			= 0,
		fVelocityMagnitude 			= 0,
		fAccelDirection 			= 0,
		fAccelMagnitude 			= 0,
		fEndHoldDuration 			= 0,
		eLocation 					= CombatFloater.CodeEnumFloaterLocation.Top,
		fOffsetDirection 			= 0,
		fOffset 					= 0,
		eCollisionMode 				= CombatFloater.CodeEnumFloaterCollisionMode.Horizontal,
		fExpandCollisionBoxWidth 	= 1,
		fExpandCollisionBoxHeight 	= 1,
		nColor 						= 0xFFFFFF,
		iUseDigitSpriteSet 			= nil,
		bUseScreenPos 				= false,
		bShowOnTop 					= false,
		fRotation 					= 0,
		fDelay 						= 0,
		nDigitSpriteSpacing 		= 0,
	}

	return tTextOption
end

function FMCT:FloaterOffsetAdjustment(PlayerY, TargetY, important)
	local MaxFloaterVertical = (Apollo.GetDisplaySize().nHeight/4)
	if TargetY <= 1 then
		--nameplate is clipped off of the screen
		if g_FMCT.config.preventOutClipping then
			return CombatFloater.CodeEnumFloaterLocation.Bottom
		else
			return CombatFloater.CodeEnumFloaterLocation.Chest
		end
	elseif TargetY < MaxFloaterVertical and TargetY > 1 then
		--nameplate is visible
		return CombatFloater.CodeEnumFloaterLocation.Chest
	else
		--nameplate is visible, and has room on top for damage
		if (important) then
			return CombatFloater.CodeEnumFloaterLocation.Top
		else
			return CombatFloater.CodeEnumFloaterLocation.Chest
		end
	end
end

function FMCT:GetDamageFlags(unitTarget, unitCaster, eDamageType, nDamage, nShieldDamaged, nAbsorptionAmount, bCritical, xLabel)
	local flags = {
		harvest    = false,
		petIn      = false,
		petOut     = false,
		crit       = false,
		shield     = false,
		absorb     = false,
		heal       = false,
		vulnerable = false,
		miss       = false,
		ccSuccess  = false,
		ccFail     = false,
		important  = false,
	}
	-- dev
	if (g_FMCT.config.debug) then
		API_LOG(tostring(unitTarget:GetType()))
	end

	-- harvest
	if (unitTarget:GetType() == "Harvest" or unitTarget:GetType() == "HousingPlant") then
		flags.harvest = true
	end
	-- crit
	if bCritical == true then
		flags.crit = true
	end
	-- absorb
	if (type(nAbsorptionAmount) == "number" and nAbsorptionAmount > 0) then
		flags.absorb = true
	end
	-- heal
	if (eDamageType == GameLib.CodeEnumDamageType.Heal) then
		flags.heal = true
	end
	-- shield
	if (eDamageType == GameLib.CodeEnumDamageType.HealShields) then
		flags.shield = true
	end
	-- vulnerable
	if (unitTarget:IsInCCState( Unit.CodeEnumCCState.Vulnerability ) or eDamageType == knTestingVulnerable) then
		flags.vulnerable = true
	end
	
	-- pet outgoing
	if unitCaster and g_FMCT:IsFriendlyPet(unitCaster) and not flags.absorb then
		flags.petOut = true
	end

	-- pet incoming
	if (g_FMCT:IsFriendlyPet(unitTarget)) then
		flags.petIn = true
	end

	-- miss
	if xLabel == "miss" then
		flags.miss      = true
		flags.important = true
	end

	-- cc succeeded / failed
	if xLabel == "ccSuccess" then
		flags.ccSuccess = true
		flags.important = true
	elseif xLabel == "ccFail" then
		flags.ccFail    = true
		flags.important = true
	end

	return flags
end

function FMCT:GetStyleByFlags(flags,inOrOut)
	local style = {}
	local tStyle = flags.crit and "crit" or "hit"

	style.constraints = g_FMCT.config.style[inOrOut].constraints[tStyle]
	if flags.harvest then
		style.color = g_FMCT.config.style[inOrOut]['harvest'][tStyle]
	elseif flags.miss then
		style.color = g_FMCT.config.style[inOrOut]['miss'][tStyle]
	elseif flags.heal then
		style.color = g_FMCT.config.style[inOrOut]['heal'][tStyle]
	elseif flags.absorb then
		style.color = g_FMCT.config.style[inOrOut]['absorb'][tStyle]
	elseif flags.vulnerable then
		style.color = g_FMCT.config.style[inOrOut]['vulnerable'][tStyle]
	elseif flags.shield then
		style.color = g_FMCT.config.style[inOrOut]['shield'][tStyle]
	else
		style.color = g_FMCT.config.style[inOrOut]['damage'][tStyle]
	end

	--pet damage coloring
	if (flags.petOut) and not flags.heal and not flags.shield then
		style.color = g_FMCT.config.style[inOrOut]['pet'][tStyle]
	end
	if (flags.petIn) and not flags.heal and not flags.shield then
		style.color = g_FMCT.config.style["incoming"]['pet'][tStyle]
	end

	--cc coloring
	if flags.ccSuccess then
		style.color = g_FMCT.config.style[inOrOut]['ccSuccess'][tStyle]
	elseif flags.ccFail then
		style.color = g_FMCT.config.style[inOrOut]['ccFail'][tStyle]
	end

	--is it important?
	if flags.important then
		style.constraints = g_FMCT.config.style[inOrOut].constraints.important
	end

	return style
end

function FMCT:IsFriendlyPet(unitTarget)
	local unitPlayer = GameLib.GetPlayerUnit()
	local friendlyPet = false

	if (unitTarget:GetDispositionTo(unitPlayer) == Unit.CodeEnumDisposition.Friendly) and (unitTarget:GetType() == "Pet" or unitTarget:GetType() == "Esper Pet") then
		friendlyPet = true
	end

	return friendlyPet
end

-----------------------------------------------------------------------------------------------
-- Uncharted
-----------------------------------------------------------------------------------------------
function API_LOG(msg)
	ChatSystemLib.PostOnChannel(2, "[FMCT]: " .. msg)
end

-----------------------------------------------------------------------------------------------
-- Addon Instance
-----------------------------------------------------------------------------------------------
local FMCTInstance = FMCT:new()
FMCTInstance:Init()