function _BuildUI(ui)
	-- grab all supported fonts ------------
	----------------------------------------
	local fonts = {}
	local fontlib = Apollo:GetGameFonts()

	--we want CRB_FloaterHuge to be at the top
	table.insert(fonts, {"CRB_FloaterHuge"})
	for k, v in pairs(fontlib) do
		if v.name ~= "CRB_FloaterHuge" then
			table.insert(fonts, {v.name})
		end
	end

	-- choice tables -----------------------
	----------------------------------------
	ui:choicetable("fonts", fonts)

	-- general settings --------------------
	----------------------------------------
	ui:category("Basic Settings")
	:header("Display")
	:check({
		map = "allowOverNameplates",
		label = "Allow combat text to overlap nameplates",
	})
	:check({
		map = "preventOutClipping",
		label = "Prevent outgoing damage from clipping off screen against large mobs",
	})
	:header("Harvesting Combat Text")
	:note("Some nodes will show damage when harvesting, which could appear as damage to mobs when striking enemies near them.")
	:check({
		map = "allowCustomHarvestText",
		label = "Replace harvesting node damage numbers with text",
	})
	:input({
		map = "customHarvestText",
		label = "Custom Harvesting Text",
	})
	:header("Bug Fixes & Tweaks")
	:note("When fighting multiple enemies, the game sometimes fails to target anything after the first one dies.")
	:check({
		map = "autoTargetOnOutgoing",
		label = "Force target subsequent enemies if I don't have a target",
	})
	:navdivider()

	-- fonts -------------------------------
	----------------------------------------
	ui:category("Outgoing Fonts")
	:header("Hit Numbers")
	:combo({
		choices = "fonts",
		map     = "style.outgoing.constraints.hit.font",
		label   = "Family",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
	})
	:input({
		map   = "style.outgoing.constraints.hit.size",
		label   = "Scale",
	})
	:input({
		map   = "style.outgoing.constraints.hit.duration",
		label   = "Duration",
	})
	:header("Crit Numbers")
	:combo({
		choices = "fonts",
		map   = "style.outgoing.constraints.crit.font",
		label   = "Family",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
	})
	:input({
		map   = "style.outgoing.constraints.crit.size",
		label   = "Scale",
	})
	:input({
		map   = "style.outgoing.constraints.crit.duration",
		label   = "Duration",
	})
	:header("Status Text")
	:note("Status text is used for all non-numerical displays (deflect, cc status, etc)")
	:combo({
		choices = "fonts",
		map   = "style.outgoing.constraints.important.font",
		label   = "Family",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
	})
	:input({
		map   = "style.outgoing.constraints.important.size",
		label   = "Scale",
	})
	:input({
		map   = "style.outgoing.constraints.important.duration",
		label   = "Duration",
	})

	:category("Incoming Fonts")
	:header("Hit Numbers")
	:combo({
		choices = "fonts",
		map   = "style.incoming.constraints.hit.font",
		label   = "Family",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
	})
	:input({
		map   = "style.incoming.constraints.hit.size",
		label   = "Scale",
	})
	:input({
		map   = "style.incoming.constraints.hit.duration",
		label   = "Duration",
	})
	:header("Crit Numbers")
	:combo({
		choices = "fonts",
		map   = "style.incoming.constraints.crit.font",
		label   = "Family",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
	})
	:input({
		map   = "style.incoming.constraints.crit.size",
		label   = "Scale",
	})
	:input({
		map   = "style.incoming.constraints.crit.duration",
		label   = "Duration",
	})
	:header("Status Text")
	:note("Status text is used for all non-numerical displays (deflect, cc status, etc)")
	:combo({
		choices = "fonts",
		map   = "style.incoming.constraints.important.font",
		label   = "Family",
		onitemadded = function(wndControl)
			wndControl:SetFont(wndControl:GetText())
		end,
	})
	:input({
		map   = "style.incoming.constraints.important.size",
		label   = "Scale",
	})
	:input({
		map   = "style.incoming.constraints.important.duration",
		label   = "Duration",
	})
	:navdivider()

	-- colors ------------------------------
	----------------------------------------

	
	:category("Outgoing Colors")
	:header("Damage")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.damage.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.damage.crit",
		label  = "Crit Color",
	})
	:header("Pet Damage")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.pet.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.pet.crit",
		label  = "Crit Color",
	})
	:header("Absorb")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.absorb.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.absorb.crit",
		label  = "Crit Color",
	})
	:header("Vulnerable")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.vulnerable.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.vulnerable.crit",
		label  = "Crit Color",
	})
	:header("Healing")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.heal.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.heal.crit",
		label  = "Crit Color",
	})
	:header("Shield Healing")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.shield.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.shield.crit",
		label  = "Crit Color",
	})
	:header("Miss")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.miss.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.miss.crit",
		label  = "Crit Color",
	})
	:header("Crowd Control Succeded")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.ccSuccess.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.ccSuccess.crit",
		label  = "Crit Color",
	})
	:header("Crowd Control Failed")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.ccFail.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.ccFail.crit",
		label  = "Crit Color",
	})
	:header("Harvest")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.harvest.hit",
		label  = "Hit Color",
	}):color({
		format = "rgba",
		alpha = true,
		map  = "style.outgoing.harvest.crit",
		label  = "Crit Color",
	})

	:category("Incoming Colors")
	:header("Damage")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.damage.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.damage.crit",
		label  = "Crit Color",
	})
	:header("Pet Damage")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.pet.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.pet.crit",
		label  = "Crit Color",
	})
	:header("Absorb")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.absorb.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.absorb.crit",
		label  = "Crit Color",
	})
	:header("Vulnerable")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.vulnerable.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.vulnerable.crit",
		label  = "Crit Color",
	})
	:header("Healing")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.heal.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.heal.crit",
		label  = "Crit Color",
	})
	:header("Shield Healing")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.shield.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.shield.crit",
		label  = "Crit Color",
	})
	:header("Miss")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.miss.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.miss.crit",
		label  = "Crit Color",
	})
	:header("Crowd Control Succeded")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.ccSuccess.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.ccSuccess.crit",
		label  = "Crit Color",
	})
	:header("Crowd Control Failed")
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.ccFail.hit",
		label  = "Hit Color",
	})
	:color({
		format = "rgba",
		alpha = true,
		map  = "style.incoming.ccFail.crit",
		label  = "Crit Color",
	})
--]]
end